/* USBprogRFM12
 *
 * Copyright (C) 2005  Benedikt Sauter
 * Copyright (C) 2008  Manuel Stahl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdlib.h>
#include <avr/io.h>
#include <stdint.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <inttypes.h>

#include "led.h"
#include "fifo.h"
#include "usbn2mc.h"
#include "usbn2mc/class/CDC/usbcdc.h"
#include "firmwarelib/avrupdate.h"

#define UARTReady()		(UCSRA & (1<<UDRE))

volatile uint8_t printLC = 0;

volatile FIFO64_t sendBuffer;
volatile FIFO64_t recvBuffer;

SIGNAL(SIG_UART_RECV)
{
	FIFO_write(recvBuffer, UDR, 64);
//	LED_toggle();
}

/*************** usb requests  **************/

// reponse for requests on interface
void USBNInterfaceRequests(DeviceRequest_t *req, EPInfo* ep) {
}

/* id need for live update of firmware */
void USBNDecodeVendorRequest(DeviceRequest_t *req) {
	switch (req->bRequest) {
		case STARTAVRUPDATE:
			avrupdate_start();
	break;
	}
}

void USB_CDC_setLineCoding(USB_CDC_LineCoding_t* lc) {
}

void USB_CDC_rxCallback(uint8_t buf[], uint8_t len) {
	uint8_t i;
	for(i=0; i<len; i++)
		FIFO_write(sendBuffer, buf[i], 64);
}

void UARTInit() {
	UBRRH = 0;
	UBRRL = 103;

	UCSRA = (1 << RXC) | (1 << TXC);
	UCSRB = (1 << RXEN) | (1 << TXEN) | (1 << RXCIE);
	UCSRC = (1 << URSEL) | (1 << UCSZ1) | (1 << UCSZ0);
}

/*************** main function  **************/

int main(void) {
	LED_init();
	LED_off();
	FIFO_init(sendBuffer);
	FIFO_init(recvBuffer);

	sei();				// activate global interrupts

	UARTInit();

	USB_CDC_init();		// setup USBN as CDC device

	USBNAddStringDescriptor("USBprog EmbeddedProjects");	// add vendor
	USBNAddStringDescriptor("USBprogRS232");				// add product

	USBNInitMC();		// start USB controller
	USBNStart();		// start device stack

	for (;;) {
		if(FIFO_available(sendBuffer) && UARTReady()) {
			LED_toggle();
			UDR = FIFO_read(sendBuffer, 64);
		}

		uint8_t i=0;
		uint8_t buf[8];
		while(FIFO_available(recvBuffer) && i < 8)
			buf[i++] = FIFO_read(recvBuffer, 64);

		if(i) USB_CDC_tx(buf, i);

//		if(printLC) {
//			LED_on();
//			USB_CDC_tx(0, 0);
//			printLC = 0;
//			_delay_ms(2000);
//			if(printLC) {
//				avrupdate_start();
//				_delay_ms(1000);
//			} else
//				LED_off();
//		}
	}
}

