/* usbcdc.h
* Copyright (C) 2005  Benedikt Sauter
* Copyright (C) 2008  Manuel Stahl
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef USBCDC_H_
#define USBCDC_H_

#include <stdint.h>

///////////////////////////////////////////////////

#define USB_VENDOR_ID	0x1781
#define USB_PRODUCT_ID	0x0c64

///////////////////////////////////////////////////
// Endpoint Mapping:
//
// EP0 in/out:		 USB         (USBN: FIFO0 TXC0 RXC0)
// EP3 interrupt in: CDC control (USBN: FIFO1 TXC1)
// EP1 bulk out:	 CDC data    (USBN: FIFO2 RXC1)
// EP1 bulk in:	 	 CDC data    (USBN: FIFO3 TXC2)
///////////////////////////////////////////////////

typedef struct usb_cdc_line_coding {
	uint32_t dwDTERrate;   //data terminal rate, in bits per second
	char     bCharFormat;  //num of stop bits (0=1, 1=1.5, 2=2)
	char     bParityType;  //parity (0=none, 1=odd, 2=even, 3=mark, 4=space)
	char     bDataBits;    //data bits (5,6,7,8 or 16)
} USB_CDC_LineCoding_t;

/**
 * Must be implemented by the application
 * @param lc the line coding parameters
 */
void USB_CDC_setLineCoding(USB_CDC_LineCoding_t* lc);

/**
 * Must be implemented by the application
 * @param buf	buffer where the received bytes are stored
 * @param len	number of bytes received
 */
void USB_CDC_rxCallback(uint8_t buf[], uint8_t len);

/**
 * Send a byte to the PC.
 * @param data	the byte to send.
 */
void USB_CDC_tx(uint8_t* data, uint8_t len);

/**
 * Configure the USBN960x to be a USB-CDC device.
 */
void USB_CDC_init(void);

#endif /* USBCDC_H_ */
