/* usbn960x.h
* Copyright (C) 2005  Benedikt Sauter
* Copyright (C) 2008  Manuel Stahl
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _USBN960X_H
#define _USBN960X_H

#define DEBUG 0

#include <stdint.h>
#include "../usbn960xreg.h"
#include "../usb11spec.h"

/*-------------------------------------------
 * type definitions
 * ------------------------------------------*/

/**
 * Callback when data was received.
 * @param count	number of bytes received
 */
typedef void (FIFOCallback)(uint8_t count);

extern uint8_t *DeviceDescriptor;
extern uint8_t *ConfigurationDescriptor;
extern uint8_t USB_numStringDescriptors;
extern USB_StringDescriptor_t StringDescriptors[];


extern FIFOCallback* RX1Callback;


/*-------------------------------------------
 * global data structs
 * ------------------------------------------*/

typedef struct devicereq {
	uint8_t  bmRequestType;
	uint8_t  bRequest;
	uint16_t wValue;
	uint16_t wIndex;
	uint16_t wLength;
} DeviceRequest_t;

typedef enum {
	EP_MEM,
	EP_FLASH,
	EP_STRING
} BufferType_t;

typedef struct epinfo {
	const uint8_t usbnData;		// data register
	const uint8_t usbnCommand;	// command register
	const uint8_t usbnControl;	// control register
	const uint8_t usbnFifoSize;	// size of the FIFO
	uint8_t DataPid;		// 0 = data0, 1 = data1
	BufferType_t bufType;	// location of buffer (MEM, FLASH)
	uint8_t bufIndex;			// index in buffer
	uint8_t bufSize;		// size of buffer
	uint8_t* buf;			// buffer
} EPInfo;

extern EPInfo EP0rx;
extern EPInfo EP0tx;

/*-------------------------------------------
 * system functions
 * ------------------------------------------*/


/// usb default requests set address
//void _USBNGetDescriptor(DeviceRequest *req);
void _USBNClearFeature(void);

/**
 * Set selected configuration.
 * Must be implemented by the application.
 */
void USBNSetConfiguration(DeviceRequest_t *req);

/**
 * Receive data from the USBN fifo.
 * Be sure to set the following EPInfo members: bufSize, buf
 */
void USBNReceive(EPInfo* ep);

/**
 * Push data into the USBN fifo.
 * Be sure to set the following EPInfo members: bufType, Index, bufSize, buf
 */
void USBNTransmit(EPInfo* ep);

/**
 * Read a byte from the USBN controller.
 * Must be implemented for the used micro controller and wiring.
 * @param Adr	the address to read from
 * @return	the byte that was read
 */
uint8_t USBNRead(uint8_t Adr);

/**
 * Write a byte to the USBN controller.
 * Must be implemented for the used micro controller and wiring.
 * @param Adr	the address to write to
 * @param Data	the byte to write
 */
void USBNWrite(uint8_t Adr,uint8_t Data);

/**
 * Write a debug string.
 * Must be implemented by the application (i.e. write to UART).
 * @param msg	the message to write
 */
void USBNDebug(char *msg);

/**
 * Decode a vender request.
 * Must be implemented depending on the application.
 * @param req	the request to decode
 */
void USBNDecodeVendorRequest(DeviceRequest_t *req);

/**
 * Decode a class request.
 * Must be implemented depending on the USB device class.
 * @param req	the request to decode
 */
void USBNDecodeClassRequest(DeviceRequest_t *req);

#endif /* __USBN960X_H__ */
